@@ -21,10 +21,11 @@ and/or Accept headers and attempts to negotiate an API controller to
 return
 """
 
+from oslo.config import cfg
+
 from glance.api import versions
 from glance.common import wsgi
 import glance.openstack.common.log as logging
-from glance.openstack.common import cfg
 
 CONF = cfg.CONF
 
@@ -82,7 +83,7 @@ class VersionNegotiationFilter(wsgi.Middleware):
         """
         if subject in ('v1', 'v1.0', 'v1.1') and CONF.enable_v1_api:
             major_version = 1
-        elif subject in ('v2', 'v2.0') and CONF.enable_v2_api:
+        elif subject in ('v2', 'v2.0', 'v2.1') and CONF.enable_v2_api:
             major_version = 2
         else:
             raise ValueError()
