@@ -32,14 +32,14 @@ import os
 import platform
 import subprocess
 import sys
-import uuid
 
-import iso8601
+from oslo.config import cfg
 from webob import exc
 
 from glance.common import exception
 import glance.openstack.common.log as logging
 
+CONF = cfg.CONF
 
 LOG = logging.getLogger(__name__)
 
@@ -256,18 +256,6 @@ def bool_from_string(subject):
     return False
 
 
-def generate_uuid():
-    return str(uuid.uuid4())
-
-
-def is_uuid_like(value):
-    try:
-        uuid.UUID(value)
-        return True
-    except Exception:
-        return False
-
-
 def safe_mkdirs(path):
     try:
         os.makedirs(path)
@@ -357,17 +345,17 @@ def get_terminal_size():
 
         try:
             height_width = struct.unpack('hh', fcntl.ioctl(sys.stderr.fileno(),
-                                        termios.TIOCGWINSZ,
-                                        struct.pack('HH', 0, 0)))
+                                         termios.TIOCGWINSZ,
+                                         struct.pack('HH', 0, 0)))
         except:
             pass
 
         if not height_width:
             try:
                 p = subprocess.Popen(['stty', 'size'],
-                                    shell=False,
-                                    stdout=subprocess.PIPE,
-                                    stderr=open(os.devnull, 'w'))
+                                     shell=False,
+                                     stdout=subprocess.PIPE,
+                                     stderr=open(os.devnull, 'w'))
                 result = p.communicate()
                 if p.returncode == 0:
                     return tuple(int(x) for x in result[0].split())
@@ -388,7 +376,7 @@ def get_terminal_size():
             import struct
             unpack_tmp = struct.unpack("hhhhHhhhhhh", csbi.raw)
             (bufx, bufy, curx, cury, wattr,
-            left, top, right, bottom, maxx, maxy) = unpack_tmp
+             left, top, right, bottom, maxx, maxy) = unpack_tmp
             height = bottom - top + 1
             width = right - left + 1
             return (height, width)
@@ -403,7 +391,7 @@ def get_terminal_size():
 
     height_width = func.get(platform.os.name, _get_terminal_size_unknownOS)()
 
-    if height_width == None:
+    if height_width is None:
         raise exception.Invalid()
 
     for i in height_width:
@@ -424,3 +412,58 @@ def mutating(func):
                                     content_type="text/plain")
         return func(self, req, *args, **kwargs)
     return wrapped
+
+
+def setup_remote_pydev_debug(host, port):
+
+        error_msg = ('Error setting up the debug environment.  Verify that the'
+                     ' option pydev_worker_debug_port is pointing to a valid '
+                     'hostname or IP on which a pydev server is listening on'
+                     ' the port indicated by pydev_worker_debug_port.')
+
+        try:
+            from pydev import pydevd
+
+            pydevd.settrace(host,
+                            port=port,
+                            stdoutToServer=True,
+                            stderrToServer=True)
+            return True
+        except:
+            LOG.exception(error_msg)
+            raise
+
+
+class LazyPluggable(object):
+    """A pluggable backend loaded lazily based on some value."""
+
+    def __init__(self, pivot, config_group=None, **backends):
+        self.__backends = backends
+        self.__pivot = pivot
+        self.__backend = None
+        self.__config_group = config_group
+
+    def __get_backend(self):
+        if not self.__backend:
+            if self.__config_group is None:
+                backend_name = CONF[self.__pivot]
+            else:
+                backend_name = CONF[self.__config_group][self.__pivot]
+            if backend_name not in self.__backends:
+                msg = _('Invalid backend: %s') % backend_name
+                raise exception.GlanceException(msg)
+
+            backend = self.__backends[backend_name]
+            if isinstance(backend, tuple):
+                name = backend[0]
+                fromlist = backend[1]
+            else:
+                name = backend
+                fromlist = backend
+
+            self.__backend = __import__(name, None, None, fromlist)
+        return self.__backend
+
+    def __getattr__(self, key):
+        backend = self.__get_backend()
+        return getattr(backend, key)
