@@ -25,14 +25,14 @@ Defines interface for DB access
 import logging
 import time
 
+from oslo.config import cfg
 import sqlalchemy
-import sqlalchemy.orm
-import sqlalchemy.sql
+import sqlalchemy.orm as sa_orm
+import sqlalchemy.sql as sa_sql
 
 from glance.common import exception
 from glance.db.sqlalchemy import migration
 from glance.db.sqlalchemy import models
-from glance.openstack.common import cfg
 import glance.openstack.common.log as os_logging
 from glance.openstack.common import timeutils
 
@@ -51,13 +51,14 @@ STATUSES = ['active', 'saving', 'queued', 'killed', 'pending_delete',
 
 db_opts = [
     cfg.IntOpt('sql_idle_timeout', default=3600),
-    cfg.IntOpt('sql_max_retries', default=10),
+    cfg.IntOpt('sql_max_retries', default=60),
     cfg.IntOpt('sql_retry_interval', default=1),
     cfg.BoolOpt('db_auto_create', default=False),
-    ]
+]
 
 CONF = cfg.CONF
 CONF.register_opts(db_opts)
+CONF.import_opt('debug', 'glance.openstack.common.log')
 
 
 def ping_listener(dbapi_conn, connection_rec, connection_proxy):
@@ -81,51 +82,28 @@ def ping_listener(dbapi_conn, connection_rec, connection_proxy):
             raise
 
 
+def setup_db_env():
+    """
+    Setup configuration for database
+    """
+    global sa_logger, _IDLE_TIMEOUT, _MAX_RETRIES, _RETRY_INTERVAL, _CONNECTION
+
+    _IDLE_TIMEOUT = CONF.sql_idle_timeout
+    _MAX_RETRIES = CONF.sql_max_retries
+    _RETRY_INTERVAL = CONF.sql_retry_interval
+    _CONNECTION = CONF.sql_connection
+    sa_logger = logging.getLogger('sqlalchemy.engine')
+    if CONF.debug:
+        sa_logger.setLevel(logging.DEBUG)
+
+
 def configure_db():
     """
     Establish the database, create an engine if needed, and
     register the models.
     """
-    global _ENGINE, sa_logger, _MAX_RETRIES, _RETRY_INTERVAL
-    if not _ENGINE:
-        sql_connection = CONF.sql_connection
-        _MAX_RETRIES = CONF.sql_max_retries
-        _RETRY_INTERVAL = CONF.sql_retry_interval
-        connection_dict = sqlalchemy.engine.url.make_url(sql_connection)
-        engine_args = {'pool_recycle': CONF.sql_idle_timeout,
-                       'echo': False,
-                       'convert_unicode': True
-                       }
-
-        try:
-            _ENGINE = sqlalchemy.create_engine(sql_connection, **engine_args)
-
-            if 'mysql' in connection_dict.drivername:
-                sqlalchemy.event.listen(_ENGINE, 'checkout', ping_listener)
-
-            _ENGINE.connect = wrap_db_error(_ENGINE.connect)
-            _ENGINE.connect()
-        except Exception, err:
-            msg = _("Error configuring registry database with supplied "
-                    "sql_connection '%(sql_connection)s'. "
-                    "Got error:\n%(err)s") % locals()
-            LOG.error(msg)
-            raise
-
-        sa_logger = logging.getLogger('sqlalchemy.engine')
-        if CONF.debug:
-            sa_logger.setLevel(logging.DEBUG)
-
-        if CONF.db_auto_create:
-            LOG.info('auto-creating glance registry DB')
-            models.register_models(_ENGINE)
-            try:
-                migration.version_control()
-            except exception.DatabaseMigrationError:
-                # only arises when the DB exists and is under version control
-                pass
-        else:
-            LOG.info('not auto-creating glance registry DB')
+    setup_db_env()
+    get_engine()
 
 
 def check_mutate_authorization(context, image_ref):
@@ -144,11 +122,72 @@ def get_session(autocommit=True, expire_on_commit=False):
     """Helper method to grab session"""
     global _MAKER
     if not _MAKER:
-        assert _ENGINE
-        _MAKER = sqlalchemy.orm.sessionmaker(bind=_ENGINE,
-                                             autocommit=autocommit,
-                                             expire_on_commit=expire_on_commit)
-    return _MAKER()
+        get_engine()
+        get_maker(autocommit, expire_on_commit)
+        assert(_MAKER)
+    session = _MAKER()
+    return session
+
+
+def get_engine():
+    """Return a SQLAlchemy engine."""
+    """May assign _ENGINE if not already assigned"""
+    global _ENGINE, sa_logger, _CONNECTION, _IDLE_TIMEOUT, _MAX_RETRIES,\
+        _RETRY_INTERVAL
+
+    if not _ENGINE:
+        tries = _MAX_RETRIES
+        retry_interval = _RETRY_INTERVAL
+
+        connection_dict = sqlalchemy.engine.url.make_url(_CONNECTION)
+
+        engine_args = {
+            'pool_recycle': _IDLE_TIMEOUT,
+            'echo': False,
+            'convert_unicode': True}
+
+        try:
+            _ENGINE = sqlalchemy.create_engine(_CONNECTION, **engine_args)
+
+            if 'mysql' in connection_dict.drivername:
+                sqlalchemy.event.listen(_ENGINE, 'checkout', ping_listener)
+
+            _ENGINE.connect = wrap_db_error(_ENGINE.connect)
+            _ENGINE.connect()
+        except Exception, err:
+            msg = _("Error configuring registry database with supplied "
+                    "sql_connection. Got error: %s") % err
+            LOG.error(msg)
+            raise
+
+        sa_logger = logging.getLogger('sqlalchemy.engine')
+        if CONF.debug:
+            sa_logger.setLevel(logging.DEBUG)
+
+        if CONF.db_auto_create:
+            LOG.info(_('auto-creating glance registry DB'))
+            models.register_models(_ENGINE)
+            try:
+                migration.version_control()
+            except exception.DatabaseMigrationError:
+                # only arises when the DB exists and is under version control
+                pass
+        else:
+            LOG.info(_('not auto-creating glance registry DB'))
+
+    return _ENGINE
+
+
+def get_maker(autocommit=True, expire_on_commit=False):
+    """Return a SQLAlchemy sessionmaker."""
+    """May assign __MAKER if not already assigned"""
+    global _MAKER, _ENGINE
+    assert _ENGINE
+    if not _MAKER:
+        _MAKER = sa_orm.sessionmaker(bind=_ENGINE,
+                                     autocommit=autocommit,
+                                     expire_on_commit=expire_on_commit)
+    return _MAKER
 
 
 def is_db_connection_error(args):
@@ -174,7 +213,7 @@ def wrap_db_error(f):
             remaining_attempts = _MAX_RETRIES
             while True:
                 LOG.warning(_('SQL connection failed. %d attempts left.'),
-                                remaining_attempts)
+                            remaining_attempts)
                 remaining_attempts -= 1
                 time.sleep(_RETRY_INTERVAL)
                 try:
@@ -209,38 +248,55 @@ def image_destroy(context, image_id):
     """Destroy the image or raise if it does not exist."""
     session = get_session()
     with session.begin():
-        image_ref = image_get(context, image_id, session=session)
+        image_ref = _image_get(context, image_id, session=session)
 
         # Perform authorization check
         check_mutate_authorization(context, image_ref)
 
+        _image_locations_set(image_ref.id, [], session)
+
         image_ref.delete(session=session)
 
         for prop_ref in image_ref.properties:
             image_property_delete(context, prop_ref, session=session)
 
-        for memb_ref in image_member_find(context, image_id=image_id):
-            image_member_delete(context, memb_ref, session=session)
+        members = _image_member_find(context, session, image_id=image_id)
+        for memb_ref in members:
+            _image_member_delete(context, memb_ref, session)
 
-        return image_ref
+    return _normalize_locations(image_ref)
+
+
+def _normalize_locations(image):
+    undeleted_locations = filter(lambda x: not x.deleted, image['locations'])
+    image['locations'] = [loc['value'] for loc in undeleted_locations]
+    return image
 
 
 def image_get(context, image_id, session=None, force_show_deleted=False):
+    image = _image_get(context, image_id, session=session,
+                       force_show_deleted=force_show_deleted)
+    image = _normalize_locations(image.to_dict())
+    return image
+
+
+def _image_get(context, image_id, session=None, force_show_deleted=False):
     """Get an image or raise if it does not exist."""
     session = session or get_session()
 
     try:
-        query = session.query(models.Image).\
-                options(sqlalchemy.orm.joinedload(models.Image.properties)).\
-                filter_by(id=image_id)
+        query = session.query(models.Image)\
+                       .options(sa_orm.joinedload(models.Image.properties))\
+                       .options(sa_orm.joinedload(models.Image.locations))\
+                       .filter_by(id=image_id)
 
         # filter out deleted images if context disallows it
-        if not force_show_deleted and not can_show_deleted(context):
+        if not force_show_deleted and not _can_show_deleted(context):
             query = query.filter_by(deleted=False)
 
         image = query.one()
 
-    except sqlalchemy.orm.exc.NoResultFound:
+    except sa_orm.exc.NoResultFound:
         raise exception.NotFound("No image found with ID %s" % image_id)
 
     # Make sure they can look at it
@@ -298,7 +354,7 @@ def is_image_sharable(context, image, **kwargs):
     return member['can_share']
 
 
-def is_image_visible(context, image):
+def is_image_visible(context, image, status=None):
     """Return True if the image is visible in this context."""
     # Is admin == image visible
     if context.is_admin:
@@ -320,9 +376,10 @@ def is_image_visible(context, image):
         # Figure out if this image is shared with that tenant
         members = image_member_find(context,
                                     image_id=image['id'],
-                                    member=context.owner)
+                                    member=context.owner,
+                                    status=status)
         if members:
-            return not members[0]['deleted']
+            return True
 
     # Private image
     return False
@@ -414,10 +471,10 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
                 raise ValueError(_("Unknown sort direction, "
                                    "must be 'desc' or 'asc'"))
 
-            criteria = sqlalchemy.sql.and_(*crit_attrs)
+            criteria = sa_sql.and_(*crit_attrs)
             criteria_list.append(criteria)
 
-        f = sqlalchemy.sql.or_(*criteria_list)
+        f = sa_sql.or_(*criteria_list)
         query = query.filter(f)
 
     if limit is not None:
@@ -427,7 +484,8 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
 
 
 def image_get_all(context, filters=None, marker=None, limit=None,
-                  sort_key='created_at', sort_dir='desc'):
+                  sort_key='created_at', sort_dir='desc',
+                  member_status='accepted'):
     """
     Get all images that match zero or more filters.
 
@@ -442,22 +500,49 @@ def image_get_all(context, filters=None, marker=None, limit=None,
     filters = filters or {}
 
     session = get_session()
-    query = session.query(models.Image).\
-            options(sqlalchemy.orm.joinedload(models.Image.properties))
+    query = session.query(models.Image)\
+                   .options(sa_orm.joinedload(models.Image.properties))\
+                   .options(sa_orm.joinedload(models.Image.locations))
 
-    if 'is_public' in filters and filters['is_public'] is not None:
-        the_filter = [models.Image.is_public == filters['is_public']]
-        if filters['is_public'] and context.owner is not None:
-            the_filter.extend([
-                (models.Image.owner == context.owner),
-                models.Image.members.any(member=context.owner, deleted=False)
-            ])
-        if len(the_filter) > 1:
-            query = query.filter(sqlalchemy.sql.or_(*the_filter))
-        else:
-            query = query.filter(the_filter[0])
+    # NOTE(markwash) treat is_public=None as if it weren't filtered
+    if 'is_public' in filters and filters['is_public'] is None:
         del filters['is_public']
 
+    if not context.is_admin:
+        visibility_filters = [models.Image.is_public == True]
+
+        if context.owner is not None:
+            if member_status == 'all':
+                visibility_filters.extend([
+                    models.Image.owner == context.owner,
+                    models.Image.members.any(member=context.owner,
+                                             deleted=False),
+                ])
+            else:
+                visibility_filters.extend([
+                    models.Image.owner == context.owner,
+                    models.Image.members.any(member=context.owner,
+                                             deleted=False,
+                                             status=member_status),
+                ])
+
+        query = query.filter(sa_sql.or_(*visibility_filters))
+
+    if 'visibility' in filters:
+        visibility = filters.pop('visibility')
+        if visibility == 'public':
+            query = query.filter(models.Image.is_public == True)
+            filters['is_public'] = True
+        elif visibility == 'private':
+            filters['is_public'] = False
+            if (not context.is_admin) and context.owner is not None:
+                query = query.filter(
+                            models.Image.owner == context.owner)
+        else:
+            query = query.filter(
+                        models.Image.members.any(member=context.owner,
+                                                 deleted=False))
+
     showing_deleted = False
     if 'changes-since' in filters:
         # normalize timestamp to UTC, as sqlalchemy doesn't appear to
@@ -503,15 +588,15 @@ def image_get_all(context, filters=None, marker=None, limit=None,
 
     marker_image = None
     if marker is not None:
-        marker_image = image_get(context, marker,
-                                 force_show_deleted=showing_deleted)
+        marker_image = _image_get(context, marker,
+                                  force_show_deleted=showing_deleted)
 
     query = paginate_query(query, models.Image, limit,
                            [sort_key, 'created_at', 'id'],
                            marker=marker_image,
                            sort_dir=sort_dir)
 
-    return query.all()
+    return [_normalize_locations(image.to_dict()) for image in query.all()]
 
 
 def _drop_protected_attrs(model_class, values):
@@ -546,7 +631,7 @@ def validate_image(values):
 
 
 def _update_values(image_ref, values):
-    for k in values.keys():
+    for k in values:
         if getattr(image_ref, k) != values[k]:
             setattr(image_ref, k, values[k])
 
@@ -570,13 +655,19 @@ def _image_update(context, values, image_id, purge_props=False):
         # not a dict.
         properties = values.pop('properties', {})
 
+        try:
+            locations = values.pop('locations')
+            locations_provided = True
+        except KeyError:
+            locations_provided = False
+
         if image_id:
-            image_ref = image_get(context, image_id, session=session)
+            image_ref = _image_get(context, image_id, session=session)
 
             # Perform authorization check
             check_mutate_authorization(context, image_ref)
         else:
-            if 'size' in values:
+            if values.get('size') is not None:
                 values['size'] = int(values['size'])
 
             if 'min_ram' in values:
@@ -617,9 +708,25 @@ def _image_update(context, values, image_id, purge_props=False):
         _set_properties_for_image(context, image_ref, properties, purge_props,
                                   session)
 
+    if locations_provided:
+        _image_locations_set(image_ref.id, locations, session)
+
     return image_get(context, image_ref.id)
 
 
+def _image_locations_set(image_id, locations, session):
+    location_refs = session.query(models.ImageLocation)\
+                           .filter_by(image_id=image_id)\
+                           .filter_by(deleted=False)\
+                           .all()
+    for location_ref in location_refs:
+        location_ref.delete(session=session)
+
+    for location in locations:
+        location_ref = models.ImageLocation(image_id=image_id, value=location)
+        location_ref.save()
+
+
 def _set_properties_for_image(context, image_ref, properties,
                               purge_props=False, session=None):
     """
@@ -640,14 +747,14 @@ def _set_properties_for_image(context, image_ref, properties,
                        'value': value}
         if name in orig_properties:
             prop_ref = orig_properties[name]
-            image_property_update(context, prop_ref, prop_values,
-                                  session=session)
+            _image_property_update(context, prop_ref, prop_values,
+                                   session=session)
         else:
             image_property_create(context, prop_values, session=session)
 
     if purge_props:
         for key in orig_properties.keys():
-            if not key in properties:
+            if key not in properties:
                 prop_ref = orig_properties[key]
                 image_property_delete(context, prop_ref, session=session)
 
@@ -658,11 +765,6 @@ def image_property_create(context, values, session=None):
     return _image_property_update(context, prop_ref, values, session=session)
 
 
-def image_property_update(context, prop_ref, values, session=None):
-    """Update an ImageProperty object"""
-    return _image_property_update(context, prop_ref, values, session=session)
-
-
 def _image_property_update(context, prop_ref, values, session=None):
     """
     Used internally by image_property_create and image_property_update
@@ -685,12 +787,29 @@ def image_property_delete(context, prop_ref, session=None):
 def image_member_create(context, values, session=None):
     """Create an ImageMember object"""
     memb_ref = models.ImageMember()
-    return _image_member_update(context, memb_ref, values, session=session)
+    _image_member_update(context, memb_ref, values, session=session)
+    return _image_member_format(memb_ref)
 
 
-def image_member_update(context, memb_ref, values, session=None):
+def _image_member_format(member_ref):
+    """Format a member ref for consumption outside of this module"""
+    return {
+        'id': member_ref['id'],
+        'image_id': member_ref['image_id'],
+        'member': member_ref['member'],
+        'can_share': member_ref['can_share'],
+        'status': member_ref['status'],
+        'created_at': member_ref['created_at'],
+        'updated_at': member_ref['updated_at']
+    }
+
+
+def image_member_update(context, memb_id, values):
     """Update an ImageMember object"""
-    return _image_member_update(context, memb_ref, values, session=session)
+    session = get_session()
+    memb_ref = _image_member_get(context, memb_id, session)
+    _image_member_update(context, memb_ref, values, session)
+    return _image_member_format(memb_ref)
 
 
 def _image_member_update(context, memb_ref, values, session=None):
@@ -703,41 +822,60 @@ def _image_member_update(context, memb_ref, values, session=None):
     return memb_ref
 
 
-def image_member_delete(context, memb_ref, session=None):
+def image_member_delete(context, memb_id, session=None):
     """Delete an ImageMember object"""
     session = session or get_session()
+    member_ref = _image_member_get(context, memb_id, session)
+    _image_member_delete(context, member_ref, session)
+
+
+def _image_member_delete(context, memb_ref, session):
     memb_ref.delete(session=session)
-    return memb_ref
 
 
-def image_member_find(context, image_id=None, member=None, session=None,
-                      show_deleted=True):
+def _image_member_get(context, memb_id, session):
+    """Fetch an ImageMember entity by id"""
+    query = session.query(models.ImageMember)
+    query = query.filter_by(id=memb_id)
+    return query.one()
+
+
+def image_member_find(context, image_id=None, member=None, status=None):
     """Find all members that meet the given criteria
 
     :param image_id: identifier of image entity
     :param member: tenant to which membership has been granted
-    :param session: an existing sqlalchemy session to use
-    :param show_deleted: set to False to override use of the context to
-                         determine if deleted memberships are included
     """
-    session = session or get_session()
+    session = get_session()
+    members = _image_member_find(context, session, image_id, member, status)
+    return [_image_member_format(m) for m in members]
 
-    # Note lack of permissions check; this function is called from
-    # is_image_visible(), so avoid recursive calls
+
+def _image_member_find(context, session, image_id=None,
+                       member=None, status=None):
     query = session.query(models.ImageMember)
+    query = query.filter_by(deleted=False)
+
+    if not context.is_admin:
+        query = query.join(models.Image)
+        filters = [
+            models.Image.owner == context.owner,
+            models.ImageMember.member == context.owner,
+        ]
+        query = query.filter(sa_sql.or_(*filters))
 
     if image_id is not None:
-        query = query.filter_by(image_id=image_id)
+        query = query.filter(models.ImageMember.image_id == image_id)
     if member is not None:
-        query = query.filter_by(member=member)
-    if not (show_deleted and can_show_deleted(context)):
-        query = query.filter_by(deleted=False)
+        query = query.filter(models.ImageMember.member == member)
+    if status is not None:
+        query = query.filter(models.ImageMember.status == status)
 
     return query.all()
 
 
 # pylint: disable-msg=C0111
-def can_show_deleted(context):
+def _can_show_deleted(context):
     """
     Calculates whether to include deleted objects based on context.
     Currently just looks for a flag called deleted in the context dict.
@@ -755,7 +893,10 @@ def image_tag_set_all(context, image_id, tags):
     tags = set(tags)
 
     tags_to_create = tags - existing_tags
-    for tag in tags_to_create:
+    #NOTE(bcwaldon): we call 'reversed' here to ensure the ImageTag.id fields
+    # will be populated in the order required to reflect the correct ordering
+    # on a subsequent call to image_tag_get_all
+    for tag in reversed(list(tags_to_create)):
         image_tag_create(context, image_id, tag, session)
 
     tags_to_delete = existing_tags - tags
@@ -774,13 +915,13 @@ def image_tag_create(context, image_id, value, session=None):
 def image_tag_delete(context, image_id, value, session=None):
     """Delete an image tag."""
     session = session or get_session()
-    query = session.query(models.ImageTag).\
-                    filter_by(image_id=image_id).\
-                    filter_by(value=value).\
-                    filter_by(deleted=False)
+    query = session.query(models.ImageTag)\
+                   .filter_by(image_id=image_id)\
+                   .filter_by(value=value)\
+                   .filter_by(deleted=False)
     try:
         tag_ref = query.one()
-    except sqlalchemy.orm.exc.NoResultFound:
+    except sa_orm.exc.NoResultFound:
         raise exception.NotFound()
 
     tag_ref.delete(session=session)
@@ -789,9 +930,9 @@ def image_tag_delete(context, image_id, value, session=None):
 def image_tag_get_all(context, image_id, session=None):
     """Get a list of tags for a specific image."""
     session = session or get_session()
-    tags = session.query(models.ImageTag).\
-                   filter_by(image_id=image_id).\
-                   filter_by(deleted=False).\
-                   order_by(sqlalchemy.asc(models.ImageTag.created_at)).\
-                   all()
+    tags = session.query(models.ImageTag)\
+                  .filter_by(image_id=image_id)\
+                  .filter_by(deleted=False)\
+                  .order_by(sqlalchemy.asc(models.ImageTag.created_at))\
+                  .all()
     return [tag['value'] for tag in tags]
