@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 OpenStack Foundation
 # All Rights Reserved.
 # Copyright 2013 IBM Corp.
@@ -29,3 +31,7 @@ def upgrade(migrate_engine):
             if table_name in tables:
                 migrate_engine.execute("ALTER TABLE %s Engine=InnoDB" %
                                        table_name)
+
+
+def downgrade(migrate_engine):
+    pass
