@@ -19,17 +19,10 @@
 Prunes the Image Cache
 """
 
-import logging
-
-from glance.image_cache import ImageCache
-
-logger = logging.getLogger(__name__)
+from glance.image_cache import base
 
 
-class Pruner(object):
-    def __init__(self, conf, **local_conf):
-        self.conf = conf
-        self.cache = ImageCache(conf)
+class Pruner(base.CacheApp):
 
     def run(self):
         self.cache.prune()
