@@ -19,9 +19,13 @@ import os
 import sys
 import time
 
+from oslo.config import cfg
+
+from glance.common import crypt
 from glance.common import exception
 from glance.common import utils
-from glance.openstack.common import cfg
+import glance.context
+import glance.domain.proxy
 from glance.openstack.common import importutils
 import glance.openstack.common.log as logging
 from glance.store import location
@@ -30,43 +34,27 @@ LOG = logging.getLogger(__name__)
 
 store_opts = [
     cfg.ListOpt('known_stores',
-                default=['glance.store.filesystem.Store',
-                         'glance.store.http.Store',
-                         'glance.store.rbd.Store',
-                         'glance.store.s3.Store',
-                         'glance.store.swift.Store',
-                        ]),
+                default=[
+                    'glance.store.filesystem.Store',
+                    'glance.store.http.Store',
+                    'glance.store.rbd.Store',
+                    'glance.store.s3.Store',
+                    'glance.store.swift.Store',
+                ]),
+    cfg.StrOpt('default_store', default='file',
+               help=_("Default scheme to use to store image data. The "
+                      "scheme must be registered by one of the stores "
+                      "defined by the 'known_stores' config option.")),
     cfg.StrOpt('scrubber_datadir',
                default='/var/lib/glance/scrubber'),
     cfg.BoolOpt('delayed_delete', default=False),
     cfg.IntOpt('scrub_time', default=0),
-    ]
+]
 
 CONF = cfg.CONF
 CONF.register_opts(store_opts)
 
 
-class ImageAddResult(object):
-
-    """
-    Class that represents the succesful result of adding
-    an image to a backend store.
-    """
-
-    def __init__(self, location, bytes_written, checksum=None):
-        """
-        Initialize the object
-
-        :param location: `glance.store.StoreLocation` object representing
-                         the location of the image in the backend store
-        :param bytes_written: Number of bytes written to store
-        :param checksum: Optional checksum of the image data
-        """
-        self.location = location
-        self.bytes_written = bytes_written
-        self.checksum = checksum
-
-
 class BackendException(Exception):
     pass
 
@@ -188,7 +176,17 @@ def create_stores():
     return store_count
 
 
-def get_store_from_scheme(context, scheme):
+def verify_default_store():
+    scheme = cfg.CONF.default_store
+    context = glance.context.RequestContext()
+    try:
+        get_store_from_scheme(context, scheme)
+    except exception.UnknownScheme:
+        msg = _("Store for scheme %s not found") % scheme
+        raise RuntimeError(msg)
+
+
+def get_store_from_scheme(context, scheme, loc=None):
     """
     Given a scheme, return the appropriate store object
     for handling that scheme.
@@ -196,11 +194,11 @@ def get_store_from_scheme(context, scheme):
     if scheme not in location.SCHEME_TO_CLS_MAP:
         raise exception.UnknownScheme(scheme=scheme)
     scheme_info = location.SCHEME_TO_CLS_MAP[scheme]
-    store = scheme_info['store_class'](context)
+    store = scheme_info['store_class'](context, loc)
     return store
 
 
-def get_store_from_uri(context, uri):
+def get_store_from_uri(context, uri, loc=None):
     """
     Given a URI, return the store object that would handle
     operations on the URI.
@@ -208,15 +206,15 @@ def get_store_from_uri(context, uri):
     :param uri: URI to analyze
     """
     scheme = uri[0:uri.find('/') - 1]
-    store = get_store_from_scheme(context, scheme)
+    store = get_store_from_scheme(context, scheme, loc)
     return store
 
 
 def get_from_backend(context, uri, **kwargs):
     """Yields chunks of data from backend specified by uri"""
 
-    store = get_store_from_uri(context, uri)
     loc = location.get_location_from_uri(uri)
+    store = get_store_from_uri(context, uri, loc)
 
     return store.get(loc)
 
@@ -224,16 +222,16 @@ def get_from_backend(context, uri, **kwargs):
 def get_size_from_backend(context, uri):
     """Retrieves image size from backend specified by uri"""
 
-    store = get_store_from_uri(context, uri)
     loc = location.get_location_from_uri(uri)
+    store = get_store_from_uri(context, uri, loc)
 
     return store.get_size(loc)
 
 
 def delete_from_backend(context, uri, **kwargs):
     """Removes chunks of data from backend specified by uri"""
-    store = get_store_from_uri(context, uri)
     loc = location.get_location_from_uri(uri)
+    store = get_store_from_uri(context, uri, loc)
 
     try:
         return store.delete(loc)
@@ -281,6 +279,8 @@ def schedule_delayed_delete_from_backend(uri, image_id, **kwargs):
                 'image_id': image_id}
         raise exception.Duplicate(msg)
 
+    if CONF.metadata_encryption_key is not None:
+        uri = crypt.urlsafe_encrypt(CONF.metadata_encryption_key, uri, 64)
     with open(file_path, 'w') as f:
         f.write('\n'.join([uri, str(int(delete_time))]))
     os.chmod(file_path, 0600)
@@ -294,12 +294,124 @@ def add_to_backend(context, scheme, image_id, data, size):
 
 def set_acls(context, location_uri, public=False, read_tenants=[],
              write_tenants=[]):
+    loc = location.get_location_from_uri(location_uri)
     scheme = get_store_from_location(location_uri)
-    store = get_store_from_scheme(context, scheme)
+    store = get_store_from_scheme(context, scheme, loc)
     try:
-        store.set_acls(location.get_location_from_uri(location_uri),
-                       public=public,
-                       read_tenants=read_tenants,
+        store.set_acls(loc, public=public, read_tenants=read_tenants,
                        write_tenants=write_tenants)
     except NotImplementedError:
         LOG.debug(_("Skipping store.set_acls... not implemented."))
+
+
+class ImageRepoProxy(glance.domain.proxy.Repo):
+
+    def __init__(self, image_repo, context, store_api):
+        self.context = context
+        self.store_api = store_api
+        proxy_kwargs = {'context': context, 'store_api': store_api}
+        super(ImageRepoProxy, self).__init__(image_repo,
+                                             item_proxy_class=ImageProxy,
+                                             item_proxy_kwargs=proxy_kwargs)
+
+    def _set_acls(self, image):
+        public = image.visibility == 'public'
+        member_ids = []
+        if image.locations and not public:
+            member_repo = image.get_member_repo()
+            member_ids = [m.member_id for m in member_repo.list()]
+        for location in image.locations:
+            self.store_api.set_acls(self.context, location, public,
+                                    read_tenants=member_ids)
+
+    def add(self, image):
+        result = super(ImageRepoProxy, self).add(image)
+        self._set_acls(image)
+        return result
+
+    def save(self, image):
+        result = super(ImageRepoProxy, self).save(image)
+        self._set_acls(image)
+        return result
+
+
+class ImageFactoryProxy(glance.domain.proxy.ImageFactory):
+    def __init__(self, factory, context, store_api):
+        proxy_kwargs = {'context': context, 'store_api': store_api}
+        super(ImageFactoryProxy, self).__init__(factory,
+                                                proxy_class=ImageProxy,
+                                                proxy_kwargs=proxy_kwargs)
+
+
+class ImageProxy(glance.domain.proxy.Image):
+
+    def __init__(self, image, context, store_api):
+        self.image = image
+        self.context = context
+        self.store_api = store_api
+        proxy_kwargs = {
+            'context': context,
+            'image': self,
+            'store_api': store_api,
+        }
+        super(ImageProxy, self).__init__(
+                image, member_repo_proxy_class=ImageMemberRepoProxy,
+                member_repo_proxy_kwargs=proxy_kwargs)
+
+    def delete(self):
+        self.image.delete()
+        if self.image.locations:
+            if CONF.delayed_delete:
+                self.image.status = 'pending_delete'
+                for location in self.image.locations:
+                    self.store_api.schedule_delayed_delete_from_backend(
+                            location, self.image.image_id)
+            else:
+                for location in self.image.locations:
+                    self.store_api.safe_delete_from_backend(
+                            location, self.context, self.image.image_id)
+
+    def set_data(self, data, size=None):
+        if size is None:
+            size = 0  # NOTE(markwash): zero -> unknown size
+        location, size, checksum = self.store_api.add_to_backend(
+                self.context, CONF.default_store,
+                self.image.image_id, utils.CooperativeReader(data), size)
+        self.image.locations = [location]
+        self.image.size = size
+        self.image.checksum = checksum
+        self.image.status = 'active'
+
+    def get_data(self):
+        if not self.image.locations:
+            raise exception.NotFound(_("No image data could be found"))
+        data, size = self.store_api.get_from_backend(self.context,
+                                                     self.image.locations[0])
+        return data
+
+
+class ImageMemberRepoProxy(glance.domain.proxy.Repo):
+    def __init__(self, repo, image, context, store_api):
+        self.repo = repo
+        self.image = image
+        self.context = context
+        self.store_api = store_api
+        super(ImageMemberRepoProxy, self).__init__(repo)
+
+    def _set_acls(self):
+        public = self.image.visibility == 'public'
+        if self.image.locations and not public:
+            member_ids = [m.member_id for m in self.repo.list()]
+            for location in self.image.locations:
+                self.store_api.set_acls(self.context, location, public,
+                                        read_tenants=member_ids)
+
+    def add(self, member):
+        result = super(ImageMemberRepoProxy, self).add(member)
+        self._set_acls()
+        return result
+
+    def remove(self, member):
+        result = super(ImageMemberRepoProxy, self).remove(member)
+        self._set_acls()
+        return result
