@@ -1,4 +1,6 @@
-# Copyright 2010-2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2010-2011 OpenStack, LLC
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,22 +15,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from unittest import mock
-
-from oslo_config import cfg
-
 from glance import context
+from glance.openstack.common import local
 from glance.tests.unit import utils as unit_utils
 from glance.tests import utils
 
-CONF = cfg.CONF
-
 
 def _fake_image(owner, is_public):
     return {
         'id': None,
         'owner': owner,
-        'visibility': 'public' if is_public else 'shared',
+        'is_public': is_public,
     }
 
 
@@ -53,7 +50,28 @@ class TestContext(utils.BaseTestCase):
         img = _fake_image(img_owner, img_public)
         ctx = context.RequestContext(**kwargs)
 
-        self.assertEqual(exp_res, self.db_api.is_image_visible(ctx, img))
+        self.assertEqual(self.db_api.is_image_visible(ctx, img), exp_res)
+
+    def do_sharable(self, exp_res, img_owner, membership=None, **kwargs):
+        """
+        Perform a context sharability test.  Creates a (fake) image
+        with the specified owner and is_public attributes, then
+        creates a context with the given keyword arguments and expects
+        exp_res as the result of an is_image_sharable() call on the
+        context.  If membership is not None, its value will be passed
+        in as the 'membership' keyword argument of
+        is_image_sharable().
+        """
+
+        img = _fake_image(img_owner, True)
+        ctx = context.RequestContext(**kwargs)
+
+        sharable_args = {}
+        if membership is not None:
+            sharable_args['membership'] = membership
+
+        output = self.db_api.is_image_sharable(ctx, img, **sharable_args)
+        self.assertEqual(exp_res, output)
 
     def test_empty_public(self):
         """
@@ -83,6 +101,15 @@ class TestContext(utils.BaseTestCase):
         """
         self.do_visible(True, 'pattieblack', False, is_admin=True)
 
+    def test_empty_shared(self):
+        """
+        Tests that an empty context (with is_admin set to False) can
+        not share an image, with or without membership.
+        """
+        self.do_sharable(False, 'pattieblack', None, is_admin=False)
+        self.do_sharable(False, 'pattieblack', _fake_membership(True),
+                         is_admin=False)
+
     def test_anon_public(self):
         """
         Tests that an anonymous context (with is_admin set to False)
@@ -111,12 +138,20 @@ class TestContext(utils.BaseTestCase):
         """
         self.do_visible(False, 'pattieblack', False)
 
+    def test_anon_shared(self):
+        """
+        Tests that an empty context (with is_admin set to True) can
+        not share an image, with or without membership.
+        """
+        self.do_sharable(False, 'pattieblack', None)
+        self.do_sharable(False, 'pattieblack', _fake_membership(True))
+
     def test_auth_public(self):
         """
         Tests that an authenticated context (with is_admin set to
         False) can access an image with is_public set to True.
         """
-        self.do_visible(True, None, True, project_id='froggy')
+        self.do_visible(True, None, True, tenant='froggy')
 
     def test_auth_public_unowned(self):
         """
@@ -124,7 +159,7 @@ class TestContext(utils.BaseTestCase):
         False) can access an image (which it does not own) with
         is_public set to True.
         """
-        self.do_visible(True, 'pattieblack', True, project_id='froggy')
+        self.do_visible(True, 'pattieblack', True, tenant='froggy')
 
     def test_auth_public_owned(self):
         """
@@ -132,14 +167,14 @@ class TestContext(utils.BaseTestCase):
         False) can access an image (which it does own) with is_public
         set to True.
         """
-        self.do_visible(True, 'pattieblack', True, project_id='pattieblack')
+        self.do_visible(True, 'pattieblack', True, tenant='pattieblack')
 
     def test_auth_private(self):
         """
         Tests that an authenticated context (with is_admin set to
         False) can access an image with is_public set to False.
         """
-        self.do_visible(True, None, False, project_id='froggy')
+        self.do_visible(True, None, False, tenant='froggy')
 
     def test_auth_private_unowned(self):
         """
@@ -147,7 +182,7 @@ class TestContext(utils.BaseTestCase):
         False) cannot access an image (which it does not own) with
         is_public set to False.
         """
-        self.do_visible(False, 'pattieblack', False, project_id='froggy')
+        self.do_visible(False, 'pattieblack', False, tenant='froggy')
 
     def test_auth_private_owned(self):
         """
@@ -155,7 +190,50 @@ class TestContext(utils.BaseTestCase):
         False) can access an image (which it does own) with is_public
         set to False.
         """
-        self.do_visible(True, 'pattieblack', False, project_id='pattieblack')
+        self.do_visible(True, 'pattieblack', False, tenant='pattieblack')
+
+    def test_auth_sharable(self):
+        """
+        Tests that an authenticated context (with is_admin set to
+        False) cannot share an image it neither owns nor is shared
+        with it.
+        """
+        self.do_sharable(False, 'pattieblack', None, tenant='froggy')
+
+    def test_auth_sharable_admin(self):
+        """
+        Tests that an authenticated context (with is_admin set to
+        True) can share an image it neither owns nor is shared with
+        it.
+        """
+        self.do_sharable(True, 'pattieblack', None, tenant='froggy',
+                         is_admin=True)
+
+    def test_auth_sharable_owned(self):
+        """
+        Tests that an authenticated context (with is_admin set to
+        False) can share an image it owns, even if it is not shared
+        with it.
+        """
+        self.do_sharable(True, 'pattieblack', None, tenant='pattieblack')
+
+    def test_auth_sharable_cannot_share(self):
+        """
+        Tests that an authenticated context (with is_admin set to
+        False) cannot share an image it does not own even if it is
+        shared with it, but with can_share = False.
+        """
+        self.do_sharable(False, 'pattieblack', _fake_membership(False),
+                         tenant='froggy')
+
+    def test_auth_sharable_can_share(self):
+        """
+        Tests that an authenticated context (with is_admin set to
+        False) can share an image it does not own if it is shared with
+        it with can_share = True.
+        """
+        self.do_sharable(True, 'pattieblack', _fake_membership(True),
+                         tenant='froggy')
 
     def test_request_id(self):
         contexts = [context.RequestContext().request_id for _ in range(5)]
@@ -166,59 +244,9 @@ class TestContext(utils.BaseTestCase):
         ctx = context.RequestContext(service_catalog=['foo'])
         self.assertEqual(['foo'], ctx.service_catalog)
 
-    def test_user_identity(self):
-        ctx = context.RequestContext(user_id="user",
-                                     project_id="tenant",
-                                     domain_id="domain",
-                                     user_domain_id="user-domain",
-                                     project_domain_id="project-domain")
-        self.assertEqual('user tenant domain user-domain project-domain',
-                         ctx.to_dict()["user_identity"])
-
-    def test_elevated(self):
-        """Make sure we get a whole admin-capable context from elevated()."""
-        ctx = context.RequestContext(service_catalog=['foo'],
-                                     user_id='dan',
-                                     project_id='openstack',
-                                     roles=['member'])
-        admin = ctx.elevated()
-        self.assertEqual('dan', admin.user_id)
-        self.assertEqual('openstack', admin.project_id)
-        self.assertEqual(sorted(['member', 'admin']),
-                         sorted(admin.roles))
-        self.assertEqual(['foo'], admin.service_catalog)
-        self.assertTrue(admin.is_admin)
-
-    def test_elevated_again(self):
-        """Make sure a second elevation looks the same."""
-        ctx = context.RequestContext(service_catalog=['foo'],
-                                     user_id='dan',
-                                     project_id='openstack',
-                                     roles=['member'])
-        admin = ctx.elevated()
-        admin = admin.elevated()
-        self.assertEqual('dan', admin.user_id)
-        self.assertEqual('openstack', admin.project_id)
-        self.assertEqual(sorted(['member', 'admin']),
-                         sorted(admin.roles))
-        self.assertEqual(['foo'], admin.service_catalog)
-        self.assertTrue(admin.is_admin)
-
-    @mock.patch('keystoneauth1.token_endpoint.Token')
-    @mock.patch('keystoneauth1.session.Session')
-    def test_get_ksa_client(self, mock_session, mock_token):
-        # Make sure we can get a keystoneauth1 client from our context
-        # with the token auth as expected.
-        ctx = context.RequestContext(auth_token='token')
-        # NOTE(danms): The auth config group and options are
-        # dynamically registered.  Tickling enough of the relevant
-        # code to make that happen would significantly inflate the
-        # amount of code here for no real gain, so we just mock the
-        # CONF object.
-        CONF.register_group(cfg.OptGroup('keystone_authtoken'))
-        with mock.patch.object(CONF, 'keystone_authtoken') as ksat:
-            ksat.identity_uri = 'http://keystone'
-            client = context.get_ksa_client(ctx)
-        self.assertEqual(mock_session.return_value, client)
-        mock_session.assert_called_once_with(auth=mock_token.return_value)
-        mock_token.assert_called_once_with('http://keystone', 'token')
+    def test_context_local_store(self):
+        if hasattr(local.store, 'context'):
+            del local.store.context
+        ctx = context.RequestContext()
+        self.assertTrue(hasattr(local.store, 'context'))
+        self.assertEqual(ctx, local.store.context)
