@@ -23,12 +23,13 @@ import StringIO
 import tempfile
 import urllib
 
+from oslo.config import cfg
 import stubout
 import swiftclient
 
+import glance.common.auth
 from glance.common import exception
-from glance.common import utils
-from glance.openstack.common import cfg
+from glance.openstack.common import uuidutils
 from glance.store import BackendException
 from glance.store.location import get_location_from_uri
 import glance.store.swift
@@ -36,7 +37,7 @@ from glance.tests.unit import base
 
 CONF = cfg.CONF
 
-FAKE_UUID = utils.generate_uuid
+FAKE_UUID = uuidutils.generate_uuid
 
 Store = glance.store.swift.Store
 FIVE_KB = (5 * 1024)
@@ -59,9 +60,12 @@ SWIFT_CONF = {'verbose': True,
 def stub_out_swiftclient(stubs, swift_store_auth_version):
     fixture_containers = ['glance']
     fixture_container_headers = {}
-    fixture_headers = {'glance/%s' % FAKE_UUID:
-                {'content-length': FIVE_KB,
-                 'etag': 'c2e5db72bd7fd153f53ede5da5a06de3'}}
+    fixture_headers = {
+        'glance/%s' % FAKE_UUID: {
+            'content-length': FIVE_KB,
+            'etag': 'c2e5db72bd7fd153f53ede5da5a06de3'
+        }
+    }
     fixture_objects = {'glance/%s' % FAKE_UUID:
                        StringIO.StringIO("*" * FIVE_KB)}
 
@@ -69,7 +73,7 @@ def stub_out_swiftclient(stubs, swift_store_auth_version):
         if container not in fixture_containers:
             msg = "No container %s found" % container
             raise swiftclient.ClientException(msg,
-                        http_status=httplib.NOT_FOUND)
+                                              http_status=httplib.NOT_FOUND)
         return fixture_container_headers
 
     def fake_put_container(url, token, container, **kwargs):
@@ -84,8 +88,9 @@ def stub_out_swiftclient(stubs, swift_store_auth_version):
         # Large object manifest...
         global SWIFT_PUT_OBJECT_CALLS
         SWIFT_PUT_OBJECT_CALLS += 1
+        CHUNKSIZE = 64 * 1024
         fixture_key = "%s/%s" % (container, name)
-        if not fixture_key in fixture_headers.keys():
+        if fixture_key not in fixture_headers:
             if kwargs.get('headers'):
                 etag = kwargs['headers']['ETag']
                 fixture_headers[fixture_key] = {'manifest': True,
@@ -93,12 +98,12 @@ def stub_out_swiftclient(stubs, swift_store_auth_version):
                 return etag
             if hasattr(contents, 'read'):
                 fixture_object = StringIO.StringIO()
-                chunk = contents.read(Store.CHUNKSIZE)
+                chunk = contents.read(CHUNKSIZE)
                 checksum = hashlib.md5()
                 while chunk:
                     fixture_object.write(chunk)
                     checksum.update(chunk)
-                    chunk = contents.read(Store.CHUNKSIZE)
+                    chunk = contents.read(CHUNKSIZE)
                 etag = checksum.hexdigest()
             else:
                 fixture_object = StringIO.StringIO(contents)
@@ -106,7 +111,7 @@ def stub_out_swiftclient(stubs, swift_store_auth_version):
             read_len = fixture_object.len
             if read_len > MAX_SWIFT_OBJECT_SIZE:
                 msg = ('Image size:%d exceeds Swift max:%d' %
-                        (read_len, MAX_SWIFT_OBJECT_SIZE))
+                       (read_len, MAX_SWIFT_OBJECT_SIZE))
                 raise swiftclient.ClientException(
                         msg, http_status=httplib.REQUEST_ENTITY_TOO_LARGE)
             fixture_objects[fixture_key] = fixture_object
@@ -118,15 +123,15 @@ def stub_out_swiftclient(stubs, swift_store_auth_version):
             msg = ("Object PUT failed - Object with key %s already exists"
                    % fixture_key)
             raise swiftclient.ClientException(msg,
-                        http_status=httplib.CONFLICT)
+                                              http_status=httplib.CONFLICT)
 
     def fake_get_object(url, token, container, name, **kwargs):
         # GET returns the tuple (list of headers, file object)
         fixture_key = "%s/%s" % (container, name)
-        if not fixture_key in fixture_headers:
+        if fixture_key not in fixture_headers:
             msg = "Object GET failed"
             raise swiftclient.ClientException(msg,
-                        http_status=httplib.NOT_FOUND)
+                                              http_status=httplib.NOT_FOUND)
 
         fixture = fixture_headers[fixture_key]
         if 'manifest' in fixture:
@@ -151,15 +156,15 @@ def stub_out_swiftclient(stubs, swift_store_auth_version):
         except KeyError:
             msg = "Object HEAD failed - Object does not exist"
             raise swiftclient.ClientException(msg,
-                        http_status=httplib.NOT_FOUND)
+                                              http_status=httplib.NOT_FOUND)
 
     def fake_delete_object(url, token, container, name, **kwargs):
         # DELETE returns nothing
         fixture_key = "%s/%s" % (container, name)
-        if fixture_key not in fixture_headers.keys():
+        if fixture_key not in fixture_headers:
             msg = "Object DELETE failed - Object does not exist"
             raise swiftclient.ClientException(msg,
-                        http_status=httplib.NOT_FOUND)
+                                              http_status=httplib.NOT_FOUND)
         else:
             del fixture_headers[fixture_key]
             del fixture_objects[fixture_key]
@@ -214,6 +219,16 @@ class SwiftTests(object):
         image_size = self.store.get_size(loc)
         self.assertEqual(image_size, 5120)
 
+    def test_get_size_with_multi_tenant_on(self):
+        """Test that single tenant uris work with multi tenant on."""
+        uri = ("swift://%s:key@auth_address/glance/%s" %
+               (self.swift_store_user, FAKE_UUID))
+        self.config(swift_store_multi_tenant=True)
+        #NOTE(markwash): ensure the image is found
+        context = glance.context.RequestContext()
+        size = glance.store.get_size_from_backend(context, uri)
+        self.assertEqual(size, 5120)
+
     def test_get(self):
         """Test a "normal" retrieval of an image in chunks"""
         uri = "swift://%s:key@auth_address/glance/%s" % (
@@ -236,8 +251,8 @@ class SwiftTests(object):
         http:// in the swift_store_auth_address config value
         """
         loc = get_location_from_uri("swift+http://%s:key@auth_address/"
-                                    "glance/%s" % (
-                self.swift_store_user, FAKE_UUID))
+                                    "glance/%s" %
+                                    (self.swift_store_user, FAKE_UUID))
         (image_swift, image_size) = self.store.get(loc)
         self.assertEqual(image_size, 5120)
 
@@ -264,7 +279,7 @@ class SwiftTests(object):
         expected_swift_size = FIVE_KB
         expected_swift_contents = "*" * expected_swift_size
         expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
-        expected_image_id = utils.generate_uuid()
+        expected_image_id = uuidutils.generate_uuid()
         loc = 'swift+https://%s:key@localhost:8080/glance/%s'
         expected_location = loc % (self.swift_store_user,
                                    expected_image_id)
@@ -317,13 +332,13 @@ class SwiftTests(object):
         }
 
         for variation, expected_location in variations.items():
-            image_id = utils.generate_uuid()
+            image_id = uuidutils.generate_uuid()
             expected_location = expected_location % (
                 self.swift_store_user, image_id)
             expected_swift_size = FIVE_KB
             expected_swift_contents = "*" * expected_swift_size
             expected_checksum = \
-                    hashlib.md5(expected_swift_contents).hexdigest()
+                hashlib.md5(expected_swift_contents).hexdigest()
 
             image_swift = StringIO.StringIO(expected_swift_contents)
 
@@ -367,7 +382,7 @@ class SwiftTests(object):
         # simply used self.assertRaises here
         exception_caught = False
         try:
-            self.store.add(utils.generate_uuid(), image_swift, 0)
+            self.store.add(uuidutils.generate_uuid(), image_swift, 0)
         except BackendException, e:
             exception_caught = True
             self.assertTrue("container noexist does not exist "
@@ -383,7 +398,7 @@ class SwiftTests(object):
         expected_swift_size = FIVE_KB
         expected_swift_contents = "*" * expected_swift_size
         expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
-        expected_image_id = utils.generate_uuid()
+        expected_image_id = uuidutils.generate_uuid()
         loc = 'swift+https://%s:key@localhost:8080/noexist/%s'
         expected_location = loc % (self.swift_store_user,
                                    expected_image_id)
@@ -422,7 +437,7 @@ class SwiftTests(object):
         expected_swift_size = FIVE_KB
         expected_swift_contents = "*" * expected_swift_size
         expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
-        expected_image_id = utils.generate_uuid()
+        expected_image_id = uuidutils.generate_uuid()
         loc = 'swift+https://%s:key@localhost:8080/glance/%s'
         expected_location = loc % (self.swift_store_user,
                                    expected_image_id)
@@ -475,7 +490,7 @@ class SwiftTests(object):
         expected_swift_size = FIVE_KB
         expected_swift_contents = "*" * expected_swift_size
         expected_checksum = hashlib.md5(expected_swift_contents).hexdigest()
-        expected_image_id = utils.generate_uuid()
+        expected_image_id = uuidutils.generate_uuid()
         loc = 'swift+https://%s:key@localhost:8080/glance/%s'
         expected_location = loc % (self.swift_store_user,
                                    expected_image_id)
@@ -585,10 +600,11 @@ class SwiftTests(object):
         Test that we can set a public read acl.
         """
         self.config(swift_store_multi_tenant=True)
+        context = glance.context.RequestContext()
+        store = Store(context)
         uri = "swift+http://storeurl/glance/%s" % FAKE_UUID
         loc = get_location_from_uri(uri)
-        self.store.multi_tenant = True
-        self.store.set_acls(loc, public=True)
+        store.set_acls(loc, public=True)
         container_headers = swiftclient.client.head_container('x', 'y',
                                                               'glance')
         self.assertEqual(container_headers['X-Container-Read'], ".r:*")
@@ -598,11 +614,12 @@ class SwiftTests(object):
         Test that we can set read acl for tenants.
         """
         self.config(swift_store_multi_tenant=True)
+        context = glance.context.RequestContext()
+        store = Store(context)
         uri = "swift+http://storeurl/glance/%s" % FAKE_UUID
         loc = get_location_from_uri(uri)
-        self.store.multi_tenant = True
         read_tenants = ['matt', 'mark']
-        self.store.set_acls(loc, read_tenants=read_tenants)
+        store.set_acls(loc, read_tenants=read_tenants)
         container_headers = swiftclient.client.head_container('x', 'y',
                                                               'glance')
         self.assertEqual(container_headers['X-Container-Read'],
@@ -613,11 +630,12 @@ class SwiftTests(object):
         Test that we can set write acl for tenants.
         """
         self.config(swift_store_multi_tenant=True)
+        context = glance.context.RequestContext()
+        store = Store(context)
         uri = "swift+http://storeurl/glance/%s" % FAKE_UUID
         loc = get_location_from_uri(uri)
-        self.store.multi_tenant = True
         read_tenants = ['frank', 'jim']
-        self.store.set_acls(loc, write_tenants=read_tenants)
+        store.set_acls(loc, write_tenants=read_tenants)
         container_headers = swiftclient.client.head_container('x', 'y',
                                                               'glance')
         self.assertEqual(container_headers['X-Container-Write'],
@@ -638,14 +656,9 @@ class TestStoreAuthV1(base.StoreClearingUnitTest, SwiftTests):
         self.config(**conf)
         super(TestStoreAuthV1, self).setUp()
         self.stubs = stubout.StubOutForTesting()
-        stub_out_swiftclient(self.stubs,
-                                     conf['swift_store_auth_version'])
+        stub_out_swiftclient(self.stubs, conf['swift_store_auth_version'])
         self.store = Store()
-
-    def tearDown(self):
-        """Clear the test environment"""
-        super(TestStoreAuthV1, self).tearDown()
-        self.stubs.UnsetAll()
+        self.addCleanup(self.stubs.UnsetAll)
 
 
 class TestStoreAuthV2(TestStoreAuthV1):
@@ -674,6 +687,235 @@ class TestStoreAuthV2(TestStoreAuthV1):
         self.assertEqual('swift', loc.store_name)
 
 
+class FakeConnection(object):
+    def __init__(self, authurl, user, key, retries=5, preauthurl=None,
+                 preauthtoken=None, snet=False, starting_backoff=1,
+                 tenant_name=None, os_options={}, auth_version="1"):
+        self.authurl = authurl
+        self.user = user
+        self.key = key
+        self.preauthurl = preauthurl
+        self.preauthtoken = preauthtoken
+        self.snet = snet
+        self.tenant_name = tenant_name
+        self.os_options = os_options
+        self.auth_version = auth_version
+
+
+class TestSingleTenantStoreConnections(base.IsolatedUnitTest):
+    def setUp(self):
+        super(TestSingleTenantStoreConnections, self).setUp()
+        self.stubs.Set(swiftclient, 'Connection', FakeConnection)
+        self.store = glance.store.swift.SingleTenantStore()
+        specs = {'scheme': 'swift',
+                 'auth_or_store_url': 'example.com/v2/',
+                 'user': 'tenant:user',
+                 'key': 'abcdefg',
+                 'container': 'cont',
+                 'obj': 'object'}
+        self.location = glance.store.swift.StoreLocation(specs)
+
+    def test_basic_connection(self):
+        connection = self.store.get_connection(self.location)
+        self.assertEqual(connection.authurl, 'https://example.com/v2/')
+        self.assertEqual(connection.auth_version, '2')
+        self.assertEqual(connection.user, 'user')
+        self.assertEqual(connection.tenant_name, 'tenant')
+        self.assertEqual(connection.key, 'abcdefg')
+        self.assertEqual(connection.snet, False)
+        self.assertEqual(connection.preauthurl, None)
+        self.assertEqual(connection.preauthtoken, None)
+        self.assertEqual(connection.os_options,
+                         {'service_type': 'object-store',
+                          'endpoint_type': 'publicURL'})
+
+    def test_connection_with_no_trailing_slash(self):
+        self.location.auth_or_store_url = 'example.com/v2'
+        connection = self.store.get_connection(self.location)
+        self.assertEqual(connection.authurl, 'https://example.com/v2/')
+
+    def test_connection_with_auth_v1(self):
+        self.config(swift_store_auth_version='1')
+        self.store.configure()
+        self.location.user = 'auth_v1_user'
+        connection = self.store.get_connection(self.location)
+        self.assertEqual(connection.auth_version, '1')
+        self.assertEqual(connection.user, 'auth_v1_user')
+        self.assertEqual(connection.tenant_name, None)
+
+    def test_connection_invalid_user(self):
+        self.store.configure()
+        self.location.user = 'invalid:format:user'
+        self.assertRaises(exception.BadStoreUri,
+                          self.store.get_connection, self.location)
+
+    def test_connection_missing_user(self):
+        self.store.configure()
+        self.location.user = None
+        self.assertRaises(exception.BadStoreUri,
+                          self.store.get_connection, self.location)
+
+    def test_connection_with_region(self):
+        self.config(swift_store_region='Sahara')
+        self.store.configure()
+        connection = self.store.get_connection(self.location)
+        self.assertEquals(connection.os_options,
+                          {'region_name': 'Sahara',
+                           'service_type': 'object-store',
+                           'endpoint_type': 'publicURL'})
+
+    def test_connection_with_service_type(self):
+        self.config(swift_store_service_type='shoe-store')
+        self.store.configure()
+        connection = self.store.get_connection(self.location)
+        self.assertEquals(connection.os_options,
+                          {'service_type': 'shoe-store',
+                           'endpoint_type': 'publicURL'})
+
+    def test_connection_with_endpoint_type(self):
+        self.config(swift_store_endpoint_type='internalURL')
+        self.store.configure()
+        connection = self.store.get_connection(self.location)
+        self.assertEquals(connection.os_options,
+                          {'service_type': 'object-store',
+                           'endpoint_type': 'internalURL'})
+
+    def test_connection_with_snet(self):
+        self.config(swift_enable_snet=True)
+        self.store.configure()
+        connection = self.store.get_connection(self.location)
+        self.assertEquals(connection.snet, True)
+
+
+class TestMultiTenantStoreConnections(base.IsolatedUnitTest):
+    def setUp(self):
+        super(TestMultiTenantStoreConnections, self).setUp()
+        self.stubs.Set(swiftclient, 'Connection', FakeConnection)
+        self.context = glance.context.RequestContext(
+                user='user', tenant='tenant', auth_tok='0123')
+        self.store = glance.store.swift.MultiTenantStore(self.context)
+        specs = {'scheme': 'swift',
+                 'auth_or_store_url': 'example.com',
+                 'container': 'cont',
+                 'obj': 'object'}
+        self.location = glance.store.swift.StoreLocation(specs)
+
+    def test_basic_connection(self):
+        self.store.configure()
+        connection = self.store.get_connection(self.location)
+        self.assertEqual(connection.authurl, None)
+        self.assertEqual(connection.auth_version, '2')
+        self.assertEqual(connection.user, 'user')
+        self.assertEqual(connection.tenant_name, 'tenant')
+        self.assertEqual(connection.key, None)
+        self.assertEqual(connection.snet, False)
+        self.assertEqual(connection.preauthurl, 'https://example.com')
+        self.assertEqual(connection.preauthtoken, '0123')
+        self.assertEqual(connection.os_options, {})
+
+    def test_connection_with_snet(self):
+        self.config(swift_enable_snet=True)
+        self.store.configure()
+        connection = self.store.get_connection(self.location)
+        self.assertEquals(connection.snet, True)
+
+
+class FakeGetEndpoint(object):
+    def __init__(self, response):
+        self.response = response
+
+    def __call__(self, service_catalog, service_type=None,
+                 endpoint_region=None, endpoint_type=None):
+        self.service_type = service_type
+        self.endpoint_region = endpoint_region
+        self.endpoint_type = endpoint_type
+        return self.response
+
+
+class TestCreatingLocations(base.IsolatedUnitTest):
+    def test_single_tenant_location(self):
+        self.config(swift_store_auth_address='example.com/v2',
+                    swift_store_container='container',
+                    swift_store_user='tenant:user',
+                    swift_store_key='auth_key')
+        store = glance.store.swift.SingleTenantStore()
+        location = store.create_location('image-id')
+        self.assertEquals(location.scheme, 'swift+https')
+        self.assertEquals(location.swift_url, 'https://example.com/v2')
+        self.assertEquals(location.container, 'container')
+        self.assertEquals(location.obj, 'image-id')
+        self.assertEquals(location.user, 'tenant:user')
+        self.assertEquals(location.key, 'auth_key')
+
+    def test_single_tenant_location_http(self):
+        self.config(swift_store_auth_address='http://example.com/v2',
+                    swift_store_container='container',
+                    swift_store_user='tenant:user',
+                    swift_store_key='auth_key')
+        store = glance.store.swift.SingleTenantStore()
+        location = store.create_location('image-id')
+        self.assertEquals(location.scheme, 'swift+http')
+        self.assertEquals(location.swift_url, 'http://example.com/v2')
+
+    def test_multi_tenant_location(self):
+        self.config(swift_store_container='container')
+        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
+        self.stubs.Set(glance.common.auth, 'get_endpoint', fake_get_endpoint)
+        context = glance.context.RequestContext(
+                user='user', tenant='tenant', auth_tok='123',
+                service_catalog={})
+        store = glance.store.swift.MultiTenantStore(context)
+        location = store.create_location('image-id')
+        self.assertEquals(location.scheme, 'swift+https')
+        self.assertEquals(location.swift_url, 'https://some_endpoint')
+        self.assertEquals(location.container, 'container_image-id')
+        self.assertEquals(location.obj, 'image-id')
+        self.assertEquals(location.user, None)
+        self.assertEquals(location.key, None)
+        self.assertEquals(fake_get_endpoint.service_type, 'object-store')
+
+    def test_multi_tenant_location_http(self):
+        fake_get_endpoint = FakeGetEndpoint('http://some_endpoint')
+        self.stubs.Set(glance.common.auth, 'get_endpoint', fake_get_endpoint)
+        context = glance.context.RequestContext(
+                user='user', tenant='tenant', auth_tok='123',
+                service_catalog={})
+        store = glance.store.swift.MultiTenantStore(context)
+        location = store.create_location('image-id')
+        self.assertEquals(location.scheme, 'swift+http')
+        self.assertEquals(location.swift_url, 'http://some_endpoint')
+
+    def test_multi_tenant_location_with_region(self):
+        self.config(swift_store_region='WestCarolina')
+        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
+        self.stubs.Set(glance.common.auth, 'get_endpoint', fake_get_endpoint)
+        context = glance.context.RequestContext(
+                user='user', tenant='tenant', auth_tok='123',
+                service_catalog={})
+        store = glance.store.swift.MultiTenantStore(context)
+        self.assertEquals(fake_get_endpoint.endpoint_region, 'WestCarolina')
+
+    def test_multi_tenant_location_custom_service_type(self):
+        self.config(swift_store_service_type='toy-store')
+        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
+        self.stubs.Set(glance.common.auth, 'get_endpoint', fake_get_endpoint)
+        context = glance.context.RequestContext(
+                user='user', tenant='tenant', auth_tok='123',
+                service_catalog={})
+        store = glance.store.swift.MultiTenantStore(context)
+        self.assertEquals(fake_get_endpoint.service_type, 'toy-store')
+
+    def test_multi_tenant_location_custom_service_type(self):
+        self.config(swift_store_endpoint_type='InternalURL')
+        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
+        self.stubs.Set(glance.common.auth, 'get_endpoint', fake_get_endpoint)
+        context = glance.context.RequestContext(
+                user='user', tenant='tenant', auth_tok='123',
+                service_catalog={})
+        store = glance.store.swift.MultiTenantStore(context)
+        self.assertEquals(fake_get_endpoint.endpoint_type, 'InternalURL')
+
+
 class TestChunkReader(base.StoreClearingUnitTest):
 
     def test_read_all_data(self):
