@@ -15,6 +15,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import socket
+import time
+
+import eventlet.patcher
 import webob
 
 from glance.common import exception
@@ -180,7 +184,7 @@ class JSONRequestDeserializerTest(test_utils.BaseTestCase):
     def test_from_json_malformed(self):
         fixture = 'kjasdklfjsklajf'
         self.assertRaises(webob.exc.HTTPBadRequest,
-                wsgi.JSONRequestDeserializer().from_json, fixture)
+                          wsgi.JSONRequestDeserializer().from_json, fixture)
 
     def test_default_no_body(self):
         request = wsgi.Request.blank('/')
@@ -243,3 +247,15 @@ class TestHelpers(test_utils.BaseTestCase):
                 self.assertEqual(v, result[k])
             else:
                 self.assertFalse(k in result)
+
+
+class TestMonkeyPatch(test_utils.BaseTestCase):
+
+    def test_time_is_monkey_patched(self):
+        """
+        Test GET with no redirect
+        """
+        server = wsgi.Server()
+        server.create_pool()
+        self.assertTrue(eventlet.patcher.is_monkey_patched(time))
+        self.assertTrue(eventlet.patcher.is_monkey_patched(socket))
