@@ -1,4 +1,4 @@
-# Copyright 2013 OpenStack Foundation
+# Copyright 2013 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,17 +14,16 @@
 #    under the License.
 
 import datetime
-import http.client as http
+import json
 
-import glance_store
-from oslo_config import cfg
-from oslo_serialization import jsonutils
+from oslo.config import cfg
 import webob
 
 import glance.api.v2.image_members
 import glance.tests.unit.utils as unit_test_utils
 import glance.tests.utils as test_utils
 
+
 DATETIME = datetime.datetime(2012, 5, 16, 15, 27, 36, 325355)
 ISOTIME = '2012-05-16T15:27:36Z'
 
@@ -50,7 +49,7 @@ def _db_fixture(id, **kwargs):
     obj = {
         'id': id,
         'name': None,
-        'visibility': 'shared',
+        'is_public': False,
         'properties': {},
         'checksum': None,
         'owner': None,
@@ -73,7 +72,6 @@ def _db_image_member_fixture(image_id, member_id, **kwargs):
     obj = {
         'image_id': image_id,
         'member': member_id,
-        'status': 'pending',
     }
     obj.update(kwargs)
     return obj
@@ -91,31 +89,25 @@ class TestImageMembersController(test_utils.BaseTestCase):
 
     def setUp(self):
         super(TestImageMembersController, self).setUp()
-        self.db = unit_test_utils.FakeDB(initialize=False)
+        self.db = unit_test_utils.FakeDB()
         self.store = unit_test_utils.FakeStoreAPI()
         self.policy = unit_test_utils.FakePolicyEnforcer()
         self.notifier = unit_test_utils.FakeNotifier()
         self._create_images()
         self._create_image_members()
-        self.controller = glance.api.v2.image_members.ImageMembersController(
-            self.db,
-            self.policy,
-            self.notifier,
-            self.store)
-        glance_store.register_opts(CONF)
-
-        self.config(default_store='filesystem',
-                    filesystem_store_datadir=self.test_dir,
-                    group="glance_store")
-
-        glance_store.create_stores()
+        self.controller = glance.api.v2.image_members\
+                                       .ImageMembersController(self.db,
+                                                               self.policy,
+                                                               self.notifier,
+                                                               self.store)
+        glance.store.create_stores()
 
     def _create_images(self):
+        self.db.reset()
         self.images = [
             _db_fixture(UUID1, owner=TENANT1, name='1', size=256,
-                        visibility='public',
-                        locations=[{'url': '%s/%s' % (BASE_URI, UUID1),
-                                    'metadata': {}, 'status': 'active'}]),
+                        is_public=True,
+                        locations=['%s/%s' % (BASE_URI, UUID1)]),
             _db_fixture(UUID2, owner=TENANT1, name='2', size=512),
             _db_fixture(UUID3, owner=TENANT3, name='3', size=512),
             _db_fixture(UUID4, owner=TENANT4, name='4', size=1024),
@@ -142,7 +134,7 @@ class TestImageMembersController(test_utils.BaseTestCase):
         actual = set([image_member.member_id
                       for image_member in output['members']])
         expected = set([TENANT4])
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_index_no_members(self):
         request = unit_test_utils.get_fake_request()
@@ -151,7 +143,7 @@ class TestImageMembersController(test_utils.BaseTestCase):
         self.assertEqual({'members': []}, output)
 
     def test_index_member_view(self):
-        # UUID3 is a shared image owned by TENANT3
+        # UUID3 is a private image owned by TENANT3
         # UUID3 has members TENANT2 and TENANT4
         # When TENANT4 lists members for UUID3, should not see TENANT2
         request = unit_test_utils.get_fake_request(tenant=TENANT4)
@@ -160,7 +152,7 @@ class TestImageMembersController(test_utils.BaseTestCase):
         actual = set([image_member.member_id
                       for image_member in output['members']])
         expected = set([TENANT4])
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_index_private_image(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT2)
@@ -179,49 +171,7 @@ class TestImageMembersController(test_utils.BaseTestCase):
         actual = set([image_member.member_id
                       for image_member in output['members']])
         expected = set([TENANT1])
-        self.assertEqual(expected, actual)
-
-    def test_index_allowed_by_get_members_policy(self):
-        rules = {"get_members": True}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request()
-        output = self.controller.index(request, UUID2)
-        self.assertEqual(1, len(output['members']))
-
-    def test_index_forbidden_by_get_members_policy(self):
-        rules = {"get_members": False}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request()
-        self.assertRaises(webob.exc.HTTPForbidden, self.controller.index,
-                          request, image_id=UUID2)
-
-    def test_show(self):
-        request = unit_test_utils.get_fake_request(tenant=TENANT1)
-        output = self.controller.show(request, UUID2, TENANT4)
-        expected = self.image_members[0]
-        self.assertEqual(expected['image_id'], output.image_id)
-        self.assertEqual(expected['member'], output.member_id)
-        self.assertEqual(expected['status'], output.status)
-
-    def test_show_by_member(self):
-        request = unit_test_utils.get_fake_request(tenant=TENANT4)
-        output = self.controller.show(request, UUID2, TENANT4)
-        expected = self.image_members[0]
-        self.assertEqual(expected['image_id'], output.image_id)
-        self.assertEqual(expected['member'], output.member_id)
-        self.assertEqual(expected['status'], output.status)
-
-    def test_show_forbidden(self):
-        request = unit_test_utils.get_fake_request(tenant=TENANT2)
-        self.assertRaises(webob.exc.HTTPNotFound, self.controller.show,
-                          request, UUID2, TENANT4)
-
-    def test_show_not_found(self):
-        # one member should not be able to view status of another member
-        # of the same image
-        request = unit_test_utils.get_fake_request(tenant=TENANT2)
-        self.assertRaises(webob.exc.HTTPNotFound, self.controller.show,
-                          request, UUID3, TENANT4)
+        self.assertEqual(actual, expected)
 
     def test_create(self):
         request = unit_test_utils.get_fake_request()
@@ -232,59 +182,6 @@ class TestImageMembersController(test_utils.BaseTestCase):
         self.assertEqual(UUID2, output.image_id)
         self.assertEqual(TENANT3, output.member_id)
 
-    def test_create_allowed_by_add_policy(self):
-        rules = {"add_member": True}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request()
-        output = self.controller.create(request, image_id=UUID2,
-                                        member_id=TENANT3)
-        self.assertEqual(UUID2, output.image_id)
-        self.assertEqual(TENANT3, output.member_id)
-
-    def test_create_forbidden_by_add_policy(self):
-        rules = {"add_member": False}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request()
-        self.assertRaises(webob.exc.HTTPForbidden, self.controller.create,
-                          request, image_id=UUID2, member_id=TENANT3)
-
-    def test_create_duplicate_member(self):
-        request = unit_test_utils.get_fake_request()
-        image_id = UUID2
-        member_id = TENANT3
-        output = self.controller.create(request, image_id=image_id,
-                                        member_id=member_id)
-        self.assertEqual(UUID2, output.image_id)
-        self.assertEqual(TENANT3, output.member_id)
-
-        self.assertRaises(webob.exc.HTTPConflict, self.controller.create,
-                          request, image_id=image_id, member_id=member_id)
-
-    def test_create_overlimit(self):
-        self.config(image_member_quota=0)
-        request = unit_test_utils.get_fake_request()
-        image_id = UUID2
-        member_id = TENANT3
-        self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
-                          self.controller.create, request,
-                          image_id=image_id, member_id=member_id)
-
-    def test_create_unlimited(self):
-        self.config(image_member_quota=-1)
-        request = unit_test_utils.get_fake_request()
-        image_id = UUID2
-        member_id = TENANT3
-        output = self.controller.create(request, image_id=image_id,
-                                        member_id=member_id)
-        self.assertEqual(UUID2, output.image_id)
-        self.assertEqual(TENANT3, output.member_id)
-
-    def test_member_create_raises_bad_request_for_unicode_value(self):
-        request = unit_test_utils.get_fake_request()
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          request, image_id=UUID5,
-                          member_id=u'\U0001f693')
-
     def test_update_done_by_member(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT4)
         image_id = UUID2
@@ -296,37 +193,17 @@ class TestImageMembersController(test_utils.BaseTestCase):
         self.assertEqual(TENANT4, output.member_id)
         self.assertEqual('accepted', output.status)
 
-    def test_update_done_by_member_forbidden_by_policy(self):
-        rules = {"modify_member": False}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request(tenant=TENANT4)
-        self.assertRaises(webob.exc.HTTPForbidden, self.controller.update,
-                          request, image_id=UUID2, member_id=TENANT4,
-                          status='accepted')
-
-    def test_update_done_by_member_allowed_by_policy(self):
-        rules = {"modify_member": True}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request(tenant=TENANT4)
-        output = self.controller.update(request, image_id=UUID2,
-                                        member_id=TENANT4,
-                                        status='accepted')
-        self.assertEqual(UUID2, output.image_id)
-        self.assertEqual(TENANT4, output.member_id)
-        self.assertEqual('accepted', output.status)
-
     def test_update_done_by_owner(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT1)
+        image_id = UUID2
+        member_id = TENANT4
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.update,
                           request, UUID2, TENANT4, status='accepted')
 
-    def test_update_non_existent_image(self):
-        request = unit_test_utils.get_fake_request(tenant=TENANT1)
-        self.assertRaises(webob.exc.HTTPNotFound, self.controller.update,
-                          request, '123', TENANT4, status='accepted')
-
     def test_update_invalid_status(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT4)
+        image_id = UUID2
+        member_id = TENANT4
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
                           request, UUID2, TENANT4, status='accept')
 
@@ -352,11 +229,10 @@ class TestImageMembersController(test_utils.BaseTestCase):
         member_id = TENANT4
         image_id = UUID2
         res = self.controller.delete(request, image_id, member_id)
-        self.assertEqual(b'', res.body)
-        self.assertEqual(http.NO_CONTENT, res.status_code)
+        self.assertEqual(res.body, '')
         found_member = self.db.image_member_find(
-            request.context, image_id=image_id, member=member_id)
-        self.assertEqual([], found_member)
+                request.context, image_id=image_id, member=member_id)
+        self.assertEqual(found_member, [])
 
     def test_delete_by_member(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT4)
@@ -368,31 +244,7 @@ class TestImageMembersController(test_utils.BaseTestCase):
         actual = set([image_member.member_id
                       for image_member in output['members']])
         expected = set([TENANT4])
-        self.assertEqual(expected, actual)
-
-    def test_delete_allowed_by_policies(self):
-        rules = {"get_member": True, "delete_member": True}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request(tenant=TENANT1)
-        output = self.controller.delete(request, image_id=UUID2,
-                                        member_id=TENANT4)
-        request = unit_test_utils.get_fake_request()
-        output = self.controller.index(request, UUID2)
-        self.assertEqual(0, len(output['members']))
-
-    def test_delete_forbidden_by_get_member_policy(self):
-        rules = {"get_member": False}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request(tenant=TENANT1)
-        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
-                          request, UUID2, TENANT4)
-
-    def test_delete_forbidden_by_delete_member_policy(self):
-        rules = {"delete_member": False}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request(tenant=TENANT1)
-        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
-                          request, UUID2, TENANT4)
+        self.assertEqual(actual, expected)
 
     def test_delete_private_image(self):
         request = unit_test_utils.get_fake_request()
@@ -416,8 +268,8 @@ class TestImageMembersController(test_utils.BaseTestCase):
         member_id = 'fake-member-id'
         image_id = UUID2
         found_member = self.db.image_member_find(
-            request.context, image_id=image_id, member=member_id)
-        self.assertEqual([], found_member)
+                request.context, image_id=image_id, member=member_id)
+        self.assertEqual(found_member, [])
         self.assertRaises(webob.exc.HTTPNotFound, self.controller.delete,
                           request, image_id, member_id)
 
@@ -462,25 +314,7 @@ class TestImageMembersSerializer(test_utils.BaseTestCase):
         response = webob.Response(request=request)
         result = {'members': self.fixtures}
         self.serializer.index(response, result)
-        actual = jsonutils.loads(response.body)
-        self.assertEqual(expected, actual)
-        self.assertEqual('application/json', response.content_type)
-
-    def test_show(self):
-        expected = {
-            'image_id': UUID2,
-            'member_id': TENANT1,
-            'status': 'accepted',
-            'created_at': ISOTIME,
-            'updated_at': ISOTIME,
-            'schema': '/v2/schemas/member',
-        }
-        request = webob.Request.blank('/v2/images/%s/members/%s'
-                                      % (UUID2, TENANT1))
-        response = webob.Response(request=request)
-        result = self.fixtures[0]
-        self.serializer.show(response, result)
-        actual = jsonutils.loads(response.body)
+        actual = json.loads(response.body)
         self.assertEqual(expected, actual)
         self.assertEqual('application/json', response.content_type)
 
@@ -496,7 +330,7 @@ class TestImageMembersSerializer(test_utils.BaseTestCase):
         response = webob.Response(request=request)
         result = self.fixtures[0]
         self.serializer.create(response, result)
-        actual = jsonutils.loads(response.body)
+        actual = json.loads(response.body)
         self.assertEqual(expected, actual)
         self.assertEqual('application/json', response.content_type)
 
@@ -512,7 +346,7 @@ class TestImageMembersSerializer(test_utils.BaseTestCase):
         response = webob.Response(request=request)
         result = self.fixtures[0]
         self.serializer.update(response, result)
-        actual = jsonutils.loads(response.body)
+        actual = json.loads(response.body)
         self.assertEqual(expected, actual)
         self.assertEqual('application/json', response.content_type)
 
@@ -525,14 +359,16 @@ class TestImagesDeserializer(test_utils.BaseTestCase):
 
     def test_create(self):
         request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes({'member': TENANT1})
+        request.body = json.dumps({'member': TENANT1})
+        image_id = UUID1
         output = self.deserializer.create(request)
         expected = {'member_id': TENANT1}
         self.assertEqual(expected, output)
 
     def test_create_invalid(self):
         request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes({'mem': TENANT1})
+        image_id = UUID1
+        request.body = json.dumps({'mem': TENANT1})
         self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
                           request)
 
@@ -541,34 +377,20 @@ class TestImagesDeserializer(test_utils.BaseTestCase):
         self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
                           request)
 
-    def test_create_member_empty(self):
-        request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes({'member': ''})
-        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
-                          request)
-
-    def test_create_list_return_error(self):
-        request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes([TENANT1])
-        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
-                          request)
-
-    def test_update_list_return_error(self):
-        request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes([TENANT1])
-        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.update,
-                          request)
-
     def test_update(self):
         request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes({'status': 'accepted'})
+        request.body = json.dumps({'status': 'accepted'})
+        image_id = UUID1
+        member_id = TENANT1
         output = self.deserializer.update(request)
         expected = {'status': 'accepted'}
         self.assertEqual(expected, output)
 
     def test_update_invalid(self):
         request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes({'mem': TENANT1})
+        image_id = UUID1
+        member_id = TENANT1
+        request.body = json.dumps({'mem': TENANT1})
         self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.update,
                           request)
 
